Player = {}
Player.__index = Player

function Player.new(map, x, y, dir_x, dir_y)

    if not dir_y then
        dir_x = dir_x or 0
        dir_y = math.sin(dir_x)
        dir_x = math.cos(dir_y)
    end

    local length = math.sqrt(dir_x*dir_x+dir_y*dir_y)
    local plane_x = (dir_y/length)*.66
    local plane_y = (dir_x/length)*.66

    return setmetatable({

        map = map;

        x = x, y = y;

        dir_x = dir_x;
        dir_y = dir_y;

        plane_x = plane_x;
        plane_y = plane_y;

        rotSpeed = 0.01;
        speed = 5;

    }, Player)

end

function Player:setMap(map)
    self.map = map
end

function Player:move(dt)

    local map = self.map

    if love.keyboard.isDown("z") then

        local move_x = self.x + self.dir_x * self.speed * dt
        if map[math.floor(self.y)][math.floor(move_x)] == 0 then self.x = move_x end

        local move_y = self.y + self.dir_y * self.speed * dt
        if map[math.floor(move_y)][math.floor(self.x)] == 0 then self.y = move_y end

    end

    if love.keyboard.isDown("s") then

        local move_x = self.x - self.dir_x * self.speed * dt
        if map[math.floor(self.y)][math.floor(move_x)] == 0 then self.x = move_x end

        local move_y = self.y - self.dir_y * self.speed * dt
        if map[math.floor(move_y)][math.floor(self.x)] == 0 then self.y = move_y end

    end

    if love.keyboard.isDown("q") then

        local old_vx = self.dir_x
        self.dir_x = self.dir_x * math.cos(-self.rotSpeed) - self.dir_y * math.sin(-self.rotSpeed)
        self.dir_y = old_vx * math.sin(-self.rotSpeed) + self.dir_y * math.cos(-self.rotSpeed)

        local old_plane_x = self.plane_x
        self.plane_x = self.plane_x * math.cos(-self.rotSpeed) - self.plane_y * math.sin(-self.rotSpeed)
        self.plane_y = old_plane_x * math.sin(-self.rotSpeed) + self.plane_y * math.cos(-self.rotSpeed)

    end

    if love.keyboard.isDown("d") then

        local old_vx = self.dir_x
        self.dir_x = self.dir_x * math.cos(self.rotSpeed) - self.dir_y * math.sin(self.rotSpeed)
        self.dir_y = old_vx * math.sin(self.rotSpeed) + self.dir_y * math.cos(self.rotSpeed)

        local old_plane_x = self.plane_x
        self.plane_x = self.plane_x * math.cos(self.rotSpeed) - self.plane_y * math.sin(self.rotSpeed)
        self.plane_y = old_plane_x * math.sin(self.rotSpeed) + self.plane_y * math.cos(self.rotSpeed)

    end

end

return Player
